body {
    font: 10pt sans-serif;
    background-color: #fff;
}
a {
    text-decoration: none;
    color: #000;
}
div.item {
    min-width: 85px;
    max-width: 110px;
    height: 150px;
    max-height: 150px;
    float: left;
    text-align: center;
    margin-left: 8px;
    margin-right: 8px;
    margin-bottom: 8px;
    position: relative;
}
div.item:hover {
    z-index: 999;
}
details {
    background-color: rgba(255,255,255,0.7);
}
details.filename {
    word-wrap: break-word;
}
details.extended {
    font-size: 8pt;
    color: #667;
    border-radius: 4px;
    min-width: 85px;
    padding: 2px;
}
img.thumb {
    box-shadow: 0px 0px 3px 2px rgba(0,0,0,0.9);
    border: 1px solid #fff;
}